# Copyright (C) 2020-2021
# Author: Cesar Roman
# Contact: cesar@thecesrom.dev

"""Vision Module.

The Vision Module brings robust SCADA visualization to Ignition, giving
you access to real-time and historical data on your HMIs and dashboard
screens.
"""
