# coding=utf-8
# Copyright (C) 2018-2021
# Author: Cesar Roman
# Contact: cesar@thecesrom.dev

"""Package information."""

__title__ = "incendium"
__description__ = "Package that extends and wraps Ignition Scripting API."
__url__ = "https://github.com/thecesrom/incendium"
__version__ = "1.0.7"
__author__ = "César Román"
__author_email__ = "cesar@thecesrom.dev"
__license__ = "MIT"
__copyright__ = "Copyright (C) 2018-2021 César Román"
